﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=9.8
@EndOfDesignText@
Sub Class_Globals
	
	Private regionRecord As regionRecord_t
	Private regionRecordHex As regionRecordHex_t
	Private fx As JFX
	Private mapDB As SQL
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize
	regionRecord.Initialize
	regionRecordHex.Initialize
	
	If initMapDatabase = False Then
		Return
	End If
End Sub



'======================================================================================================
private Sub initMapDatabase() As Boolean

	If File.Exists(File.DirApp, "uVisionMapDB.sqlite3") Then
		Try
			mapDB.InitializeSQLite(File.DirApp, "uVisionMapDB.sqlite3", False)
			'   				  logOn, setJournal, integrityCheck, doFullCheckPoint, closeAfterCheckpoint, setAutoCheckPoint) As Boolean
			If checkMapDB(mapDB, True,  "WAL",      True,           False, False,                5, File.DirApp, "uVisionMapDB.sqlite3") = False Then
				Return False
			End If
		Catch
			Log(LastException)
			Return False
		End Try
	Else
		Try
			mapDB.InitializeSQLite(File.DirApp, "uVisionMapDB.sqlite3", True)
			If checkMapDB(mapDB, True,  "WAL",      True,           False, False,                5, File.DirApp, "uVisionMapDB.sqlite3") = False Then
				Return False
			End If
		Catch
			Log(LastException)
			Return False
		End Try
		
		Try
			'Create the recent files table
			createRegionTable(mapDB)

			'   				  logOn, setJournal, integrityCheck, doFullCheckPoint, closeAfterCheckpoint, setAutoCheckPoint) As Boolean
			If checkMapDB(mapDB, True,  "WAL",      False,          False, False,   			 5, File.DirApp, "uVisionMapDB.sqlite3") = False Then
				Return False
			End If
		Catch
			Log(LastException)
			Return False
		End Try
	End If
	Return True
End Sub


'======================================================================================================
Public Sub RemoveRedundantSpace(TextToClean As String) As String
	Return Regex.Replace("[\s]{2,}", TextToClean, " ")
End Sub


'======================================================================================================
Public Sub checkMapDB(dbArg As SQL, logOn As Boolean, setJournal As String, _
							  integrityCheck As Boolean, doFullCheckPoint As Boolean, _
							  closeAfterCheckPoint As Boolean, _
							  setAutoCheckPoint As Int, path As String, dbName As String _
							  ) As Boolean
	'Check the database
	Dim qryStr As String
	Dim qryOutList As List
	Dim row() As String
	
	If integrityCheck Then
		qryStr = "PRAGMA integrity_check;"
		qryOutList = ExecuteMemoryTable(dbArg, qryStr, Null, 2)
		row = qryOutList.Get(0)
		If row(0).CompareTo("ok") <> 0 Then
			If logOn Then Log("DB Integrity = BAD")
			Return False
		End If
	
		If logOn Then Log("DB Integrity = OK")
	End If
	
	
	If setJournal <> "" Then
		qryStr = "PRAGMA journal_mode=" &  setJournal & ";"
		qryOutList = ExecuteMemoryTable(dbArg, qryStr, Null, 2)
		row = qryOutList.Get(0)
		If row(0).CompareTo("wal") <> 0 Then
			If logOn Then Log("Set Journal Mode = " &  setJournal & " Mode = FAILED")
			Return False
		End If
		
		If logOn Then Log("Set Journal Mode = " &  setJournal & " Mode = OK")
	End If
	
	
	If doFullCheckPoint Then
		dbArg.Close
		dbArg.InitializeSQLite(path, dbName, False)

		qryStr = "PRAGMA wal_checkpoint(TRUNCATE);"
		Try
			dbArg.ExecNonQuery(qryStr)
		Catch
			Log(LastException)
			If logOn Then Log("Checkpoint TRUNCATE = Failed")
			Return False
		End Try

		If logOn Then Log("Checkpoint TRUNCATE = OK")
		If closeAfterCheckPoint = True Then
			dbArg.Close
		End If
	End If

	If setAutoCheckPoint <> -3 Then
		Select setAutoCheckPoint
			Case -1
				qryStr = "PRAGMA wal_autocheckpoint=-1;" 	'No checkpoint
			Case 0
				qryStr = "PRAGMA wal_autocheckpoint=1000;"  'sqLite Default
			Case Else
				qryStr = "PRAGMA wal_autocheckpoint=" & setAutoCheckPoint & ";" 'User setting
		End Select
			
		qryOutList = ExecuteMemoryTable(dbArg, qryStr, Null, 2)
		row = qryOutList.Get(0)
		If row(0) < 0 Then
			If logOn Then Log("DB Checkpoint Value Setting = Failed")
			Return False
		End If
		
		If logOn Then Log("DB Checkpoint Value Setting = OK")
	End If
	Return True
End Sub

'======================================================================================================
Public Sub dbRecentCloseAndCheckpoint
	'   				  			 logOn, setJournal, integrityCheck, doFullCheckPoint, closeAfterCheckpoint, setAutoCheckPoint) As Boolean
	If checkMapDB(mapDB, True,  "",         True,           True,             True,                 -3, File.DirApp, "uVisionMapDB.sqlite3") = False Then
		'TODO
	End If
End Sub


'======================================================================================================
'Executes the query and returns the result as a list of arrays.
'Each item in the list is a strings array.
'StringArgs - Values to replace question marks in the query. Pass Null if not needed.
'Limit - Limits the results. Pass 0 for all results.
Public Sub ExecuteMemoryTable(SQL As SQL, Query As String, StringArgs() As String, Limit As Int) As List
	Dim cur As ResultSet

	If StringArgs = Null Then
		Dim StringArgs(0) As String
	End If
	
	Try
		cur = SQL.ExecQuery2(Query, StringArgs)
	Catch
		Log(LastException)
		Return Null
	End Try
	
'	Log("ExecuteMemoryTable: " & Query)
	Dim table As List
	table.Initialize
	
	Try
		Do While cur.NextRow
			Dim values(cur.ColumnCount) As String
			For col = 0 To cur.ColumnCount - 1
				values(col) = cur.GetString2(col)
			Next
			table.Add(values)
			If Limit > 0 And table.Size >= Limit Then Exit
		Loop
	Catch
		Log(LastException)
		Return Null
	End Try

	cur.Close
	Return table
End Sub


'======================================================================================================
Private Sub createRegionTable(db As SQL)
	Dim qryStr As String

	' Create the main Table
	qryStr = _
	$"CREATE TABLE tblRegionMap (
    	fullPathToMap    TEXT (256),
		RegionName		 TEXT,
    	unixEpoch        BIGINT,
    	dateTimeUTC      DATETIME,
    	FileCreationDate DATETIME,
    	FileModDate      DATETIME,
    	ExecBase         BIGINT,
    	LoadBase         BIGINT,
    	EndAddress       BIGINT,
    	RegionSize       BIGINT,
    	RegionFree       BIGINT,
   	 	RegionUsed       BIGINT,
    	UsagePercent     DOUBLE
	);"$
	
	Try
		db.ExecNonQuery(qryStr)
	Catch
		Log(LastException)
		Return
	End Try

	qryStr = _
	$"CREATE INDEX indxFileName ON tblRegionMap (
    fullPathToMap,
	RegionName,
    FileModDate
	);"$
	
	Try
		db.ExecNonQuery(qryStr)
	Catch
		Log(LastException)
		Return
	End Try
End Sub


'======================================================================================================
Public Sub parseMap(pathName As String, fileName As String) As List
	'See if the map file exists
	Dim listOut As List
	listOut.Initialize

	If File.Exists(pathName, fileName) = False Then
		'Error todo
		Return listOut
	End If
	
	' Clear the region map table
	If clearRegionMapTable = -1 Then
		Return listOut
	End If
	
	Dim tr As TextReader
	tr.Initialize(File.OpenInput(pathName, fileName))
	Dim lineCounter As Int
	Dim line As String = tr.ReadLine
	Do While line <> Null
		lineCounter = lineCounter + 1
		Dim m As Matcher = Regex.Matcher("Execution Region", line)
		If m.Find Then
			setRegionName(line)
			getHex32BitNumbers(line, 0)
			If addRegionToDB = -1 Then
				Return listOut
			End If
		End If
		
		If lineCounter Mod 10000000 = 0 Then
			Return listOut
		End If
		line = tr.ReadLine
	Loop
	tr.Close
	
	' Get data from db
	Return getRegionMap
	
End Sub


'======================================================================================================
private Sub setRegionName(lineStr As String) As Int
	Dim si As Int 'Start Index = 0
	Dim ei As Int 'End index
	Dim tempStr As String
	
	si = lineStr.IndexOf2("Execution Region", si)
	si = si + 17
	ei = lineStr.IndexOf2(" (", si)
	
	If ei - si < 2 Or ei - si > 120 Then
		Return -1
	End If
	
	tempStr = lineStr.SubString2(si, ei)
	regionRecord.regionName = tempStr
	regionRecordHex.regionName = tempStr
	Return 0
End Sub

'======================================================================================================
Private Sub getHex32BitNumbers(lineStr As String, startCol As Int) As Int
	Dim si As Int 'Start Index = 0
	Dim tempStr As String
	
	si = startCol
	
	Try
		si = lineStr.IndexOf2("0x", si)
		tempStr = lineStr.SubString2(si + 2, si + 10)
		tempStr = tempStr.ToUpperCase
		regionRecord.ExecBase = HexToLongInt(tempStr)
		regionRecordHex.ExecBase = 	"0x" & tempStr
	Catch
		Return -1
	End Try
	
	Try
		si = si + 12	
		si = lineStr.IndexOf2("0x", si)
		tempStr = lineStr.SubString2(si + 2, si + 10)
		tempStr = tempStr.ToUpperCase
		regionRecord.LoadBase = HexToLongInt(tempStr)
		regionRecordHex.LoadBase = 	"0x" & tempStr
	Catch
		Return -2
	End Try
	
	Try
		si = si + 12
		si = lineStr.IndexOf2("0x", si)
		tempStr = lineStr.SubString2(si + 2, si + 10)
		tempStr = tempStr.ToUpperCase
		regionRecord.RegionUsed = HexToLongInt(tempStr)
		regionRecordHex.RegionUsed = "0x" & tempStr
	Catch
		Return -3
	End Try
		
	Try	
		si = si + 12
		si = lineStr.IndexOf2("0x", si)
		tempStr = lineStr.SubString2(si + 2, si + 10)
		tempStr = tempStr.ToUpperCase
		regionRecord.RegionMax = HexToLongInt(tempStr)
		regionRecordHex.RegionMax = "0x" & tempStr
	Catch
		Return -4
	End Try
	
	Try
		regionRecord.EndAddress = regionRecord.ExecBase + regionRecord.RegionMax
		regionRecord.RegionFree = regionRecord.RegionMax - regionRecord.RegionUsed
		
		regionRecord.UsagePercent = regionRecord.RegionUsed / regionRecord.RegionMax * 100
		
		regionRecordHex.EndAddress  = LongIntToHex(regionRecord.EndAddress)
		regionRecordHex.RegionUsed  = LongIntToHex(regionRecord.RegionUsed)
		regionRecordHex.RegionFree  = LongIntToHex(regionRecord.RegionFree)
	Catch
		Return -4
	End Try
	Return 0
End Sub


'======================================================================================================
Private Sub HexToLongInt(hexStr As String) As Long
	Dim RegionUsed As Long

	RegionUsed = Bit.ParseLong(hexStr, 16)
	Return RegionUsed
End Sub


'======================================================================================================
Private Sub LongIntToHex(lint As Long) As String
	Dim RegionUsedHex As String
  
	RegionUsedHex = Bit.ToHexStringLong(lint)
	Return "0x" & RegionUsedHex
End Sub


'======================================================================================================
private Sub addRegionToDB As Int
	' Populate the meta data fields
	regionRecord.fullPathToMap = Main.pathNameSel & "\" & Main.fileNameSel
	regionRecord.unixEpoch = DateTime.Now
	regionRecord.FileCreationDate 	= Main.fAttrDates.creationDate
	regionRecord.FileModDate 		= Main.fAttrDates.LastModifiedTime
	
	If insertRegionRecordDB = -1 Then
		Return -1
	End If
	Return 0
End Sub


'======================================================================================================
Private Sub clearRegionMapTable As Int	
	Dim qryStr As String
	
	qryStr = _
	$"DELETE FROM tblRegionMap"$

	mapDB.BeginTransaction
	Try
		mapDB.ExecNonQuery(qryStr)
	Catch
		Return -1
	End Try
	mapDB.TransactionSuccessful
	Return 0
End Sub	

'======================================================================================================
private Sub insertRegionRecordDB As Int
	Dim qryStr As String
	DateTime.DateFormat = "yyyy-MM-dd"
	DateTime.TimeFormat = "HH:mm:ss"
	
	qryStr = _
	$"INSERT into tblRegionMap
	(
		fullPathToMap,
		unixEpoch,
		dateTimeUTC,
		FileCreationDate,
		FileModDate,
		ExecBase,
		LoadBase,
		EndAddress,
		RegionSize,
		RegionFree,
		RegionUsed,
		UsagePercent
	)
	VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)"$
	
	qryStr = RemoveRedundantSpace(qryStr)

	mapDB.BeginTransaction
	Try
		mapDB.ExecNonQuery2(qryStr, Array As String( _
		regionRecord.fullPathToMap, _
		regionRecord.unixEpoch, _
		regionRecord.dateTimeUTC = DateTime.Date(regionRecord.unixEpoch) & " " &  DateTime.Time(regionRecord.unixEpoch), _
		regionRecord.FileCreationDate, _
		regionRecord.FileModDate, _
		regionRecord.ExecBase, _
		regionRecord.LoadBase, _
		regionRecord.EndAddress, _
		regionRecord.RegionMax, _
		regionRecord.RegionFree, _
		regionRecord.RegionUsed, _
		regionRecord.UsagePercent))
	Catch
		Return -1
	End Try
	mapDB.TransactionSuccessful
	Return 0
End Sub


'======================================================================================================
Private Sub getRegionMap As List
	Dim qryStr As String
	
	qryStr = _
	$"SELECT 
		fullPathToMap,
		unixEpoch,
		dateTimeUTC,
		FileCreationDate,
		FileModDate,
		ExecBase,
		LoadBase,
		EndAddress,
		RegionSize,
		RegionFree,
		RegionUsed,
		UsagePercent
	FROM tblRegionMap
	ORDER BY rowid"$
	
	qryStr = RemoveRedundantSpace(qryStr)
	
	Try
		Return ExecuteMemoryTable(mapDB, qryStr, Null, 0)
	Catch
		Return Null
	End Try
	Return Null
End Sub

